/*
 * $Id: actor.d,v 1.1.1.1 2004/11/10 13:45:22 kenta Exp $
 *
 * Copyright 2004 Kenta Cho. Some rights reserved.
 */
module abagames.util.actor;

/**
 * Actor in the game that has the interface to move and draw.
 */
public class Actor {
 private:
  bool _isExist;

  public bool isExist() {
    return _isExist;
  }

  public bool isExist(bool value) {
    return _isExist = value;
  }

  public abstract void init(Object[] args);
  public abstract void move();
  public abstract void draw();
}

/**
 * Object pooling for actors.
 */
public class ActorPool(T) {
 public:
  T[] actor;
 protected:
  int actorIdx;
 private:

  public this() {}

  public this(int n, Object[] args = null) {
    createActors(n, args);
  }

  protected void createActors(int n, Object[] args = null) {
    actor = new T[n];
    foreach (inout T a; actor) {
      a = new T;
      a.isExist = false;
      a.init(args);
    }
    actorIdx = n;
  }

  public T getInstance() {
    for (int i = 0; i < actor.length; i++) {
      actorIdx--;
      if (actorIdx < 0)
	actorIdx = actor.length - 1;
      if (!actor[actorIdx].isExist) 
	return actor[actorIdx];
    }
    return null;
  }

  public T getInstanceForced() {
    actorIdx--;
    if (actorIdx < 0)
      actorIdx = actor.length - 1;
    return actor[actorIdx];
  }

  public T[] getMultipleInstances(int n) {
    T[] rsl;
    for (int i = 0; i < n; i++) {
      T inst = getInstance();
      if (!inst) {
        foreach (T r; rsl)
          r.isExist = false;
        return null;
      }
      inst.isExist = true;
      rsl ~= inst;
    }
    foreach (T r; rsl)
      r.isExist = false;
    return rsl;
  }
  
  public void move() {
    foreach (T ac; actor)
      if (ac.isExist)
	ac.move();
  }

  public void draw() {
    foreach (T ac; actor)
      if (ac.isExist)
	ac.draw();
  }

  public void clear() {
    foreach (T ac; actor)
      ac.isExist = false;
  }
}
